@@ -230,7 +230,7 @@ gdm_choose_indirect_lookup (struct sockaddr_in *clnt_sa)
 	if (id->acctime > 0 &&
 	    curtime > id->acctime + GdmMaxIndirectWait)	{
 	    gdm_debug ("gdm_choose_indirect_check: Disposing stale INDIRECT query from %s",
-		       inet_ntoa (clnt_sa->sin_addr));
+		       inet_ntoa (id->dsp_sa->sin_addr));
 	    gdm_choose_indirect_dispose (id);
 	    continue;
 	}
