@@ -30,6 +30,8 @@
 #include <sys/wait.h>
 #include <fcntl.h>
 #include <sys/stat.h>
+#include <pwd.h>
+#include <sys/types.h>
 #include "gdm.h"
 #include "misc.h"
 #include "auth.h"
@@ -49,12 +51,17 @@ extern gchar *GdmServAuthDir;
 extern gchar *GdmGtkRC;
 extern uid_t GdmUserId;
 extern gid_t GdmGroupId;
+extern gboolean GdmAddGtkModules;
+extern gchar *GdmGtkModulesList;
 
 static int screenx = 0;
 static int screeny = 0;
 static int screenwidth = 0;
 static int screenheight = 0;
 
+static gboolean inhibit_gtk_modules = FALSE;
+static gboolean inhibit_gtk_themes = FALSE;
+
 static void
 setup_cursor (GdkCursorType type)
 {
@@ -186,10 +193,11 @@ get_error_text_view (const char *details)
 }
 
 static void
-setup_dialog (GdmDisplay *d, const char *name, int closefdexcept, gboolean set_ids)
+setup_dialog (GdmDisplay *d, const char *name, int closefdexcept, gboolean set_gdm_ids, uid_t uid)
 {
 	int argc = 1;
 	char **argv;
+	struct passwd *pw;
 
 	closelog ();
 
@@ -201,10 +209,13 @@ setup_dialog (GdmDisplay *d, const char *name, int closefdexcept, gboolean set_i
 	gdm_open_dev_null (O_RDWR); /* open stdout - fd 1 */
 	gdm_open_dev_null (O_RDWR); /* open stderr - fd 2 */
 
-	if (set_ids) {
+	if (set_gdm_ids) {
 		setgid (GdmGroupId);
 		initgroups (GdmUser, GdmGroupId);
 		setuid (GdmUserId);
+		pw = NULL;
+	} else {
+		pw = getpwuid (uid);
 	}
 
 	gdm_desetuid ();
@@ -226,23 +237,56 @@ setup_dialog (GdmDisplay *d, const char *name, int closefdexcept, gboolean set_i
 	/* sanity env stuff */
 	ve_setenv ("SHELL", "/bin/sh", TRUE);
 	/* set HOME to /, we don't need no stinking HOME anyway */
-	if (set_ids)
-		ve_setenv ("HOME", "/", TRUE);
-	else
+	if (pw == NULL ||
+	    ve_string_empty (pw->pw_dir))
 		ve_setenv ("HOME", ve_sure_string (GdmServAuthDir), TRUE);
+	else
+		ve_setenv ("HOME", pw->pw_dir, TRUE);
 
-	argv = g_new0 (char *, 2);
+	argv = g_new0 (char *, 3);
 	argv[0] = (char *)name;
+	argc = 1;
+
+	if ( ! inhibit_gtk_modules &&
+	    GdmAddGtkModules &&
+	     ! ve_string_empty (GdmGtkModulesList)) {
+		argv[1] = g_strdup_printf ("--gtk-module=%s", GdmGtkModulesList);
+		argc = 2;
+	}
+
+	if (inhibit_gtk_modules) {
+		ve_unsetenv ("GTK_MODULES");
+	}
 
 	gtk_init (&argc, &argv);
 
-	if( ! ve_string_empty (GdmGtkRC) &&
+	if( ! inhibit_gtk_themes &&
+	    ! ve_string_empty (GdmGtkRC) &&
 	   access (GdmGtkRC, R_OK) == 0)
 		gtk_rc_parse (GdmGtkRC);
 
 	get_screen_size (d);
 }
 
+static gboolean
+dialog_failed (int status)
+{
+	if (WIFSIGNALED (status) &&
+	    (WTERMSIG (status) == SIGTERM ||
+	     WTERMSIG (status) == SIGINT ||
+	     WTERMSIG (status) == SIGQUIT ||
+	     WTERMSIG (status) == SIGHUP)) {
+		return FALSE;
+	} else if (WIFEXITED (status) &&
+		   WEXITSTATUS (status) == 0) {
+		return FALSE;
+	} else {
+		gdm_error ("failsafe dialog failed (inhibitions: %d %d)",
+			   inhibit_gtk_modules, inhibit_gtk_themes);
+		return TRUE;
+	}
+}
+
 void
 gdm_error_box_full (GdmDisplay *d, GtkMessageType type, const char *error,
 		    const char *details_label, const char *details_file,
@@ -270,7 +314,7 @@ gdm_error_box_full (GdmDisplay *d, GtkMessageType type, const char *error,
 		}
 
 		/* First read the details if they exist */
-		if (details_file) {
+		if (details_label != NULL && details_file != NULL) {
 			FILE *fp;
 			struct stat s;
 			int r;
@@ -323,7 +367,8 @@ gdm_error_box_full (GdmDisplay *d, GtkMessageType type, const char *error,
 		}
 
 		setup_dialog (d, "gtk-error-box", -1,
-			      (uid == 0 || uid == GdmUserId) /* set_ids */);
+			      (uid == 0 || uid == GdmUserId) /* set_gdm_ids */,
+			      uid);
 
 		loc = gdm_locale_to_utf8 (error);
 
@@ -334,6 +379,7 @@ gdm_error_box_full (GdmDisplay *d, GtkMessageType type, const char *error,
 					      "%s",
 					      loc);
 		g_free (loc);
+		gtk_widget_set_events (dlg, GDK_ALL_EVENTS_MASK);
 		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 
 		if (details_label != NULL) {
@@ -397,7 +443,24 @@ gdm_error_box_full (GdmDisplay *d, GtkMessageType type, const char *error,
 
 		_exit (0);
 	} else if (pid > 0) {
-		gdm_wait_for_extra (NULL);
+		int status;
+		gdm_wait_for_extra (&status);
+
+		if (dialog_failed (status)) {
+			if ( ! inhibit_gtk_themes) {
+				/* on failure try again, this time without any themes
+				   which may be causing a crash */
+				inhibit_gtk_themes = TRUE;
+				gdm_error_box_full (d, type, error, details_label, details_file, uid, gid);
+				inhibit_gtk_themes = FALSE;
+			} else if ( ! inhibit_gtk_modules) {
+				/* on failure try again, this time without any modules
+				   which may be causing a crash */
+				inhibit_gtk_modules = TRUE;
+				gdm_error_box_full (d, type, error, details_label, details_file, uid, gid);
+				inhibit_gtk_modules = FALSE;
+			}
+		}
 	} else {
 		gdm_error (_("%s: Cannot fork to display error/info box"),
 			   "gdm_error_box");
@@ -428,7 +491,7 @@ gdm_failsafe_question (GdmDisplay *d,
 	pid_t pid;
 	int p[2];
 
-	if (pipe (p) < 0)
+	if G_UNLIKELY (pipe (p) < 0)
 		return NULL;
 
 	pid = gdm_fork_extra ();
@@ -437,7 +500,7 @@ gdm_failsafe_question (GdmDisplay *d,
 		GtkWidget *dlg, *label, *entry;
 		char *loc;
 
-		setup_dialog (d, "gtk-failsafe-question", p[1], TRUE /* set_ids */);
+		setup_dialog (d, "gtk-failsafe-question", p[1], TRUE /* set_gdm_ids */, 0);
 
 		loc = gdm_locale_to_utf8 (question);
 
@@ -447,6 +510,7 @@ gdm_failsafe_question (GdmDisplay *d,
 						   GTK_STOCK_OK,
 						   GTK_RESPONSE_OK,
 						   NULL);
+		gtk_widget_set_events (dlg, GDK_ALL_EVENTS_MASK);
 		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		g_signal_connect (G_OBJECT (dlg), "delete_event",
 				  G_CALLBACK (gtk_true), NULL);
@@ -504,12 +568,32 @@ gdm_failsafe_question (GdmDisplay *d,
 
 		_exit (0);
 	} else if (pid > 0) {
+		int status;
 		char buf[BUFSIZ];
 		int bytes;
 
 		IGNORE_EINTR (close (p[1]));
 
-		gdm_wait_for_extra (NULL);
+		gdm_wait_for_extra (&status);
+
+		if (dialog_failed (status)) {
+			char *ret = NULL;
+			IGNORE_EINTR (close (p[0]));
+			if ( ! inhibit_gtk_themes) {
+				/* on failure try again, this time without any themes
+				   which may be causing a crash */
+				inhibit_gtk_themes = TRUE;
+				ret = gdm_failsafe_question (d, question, echo);
+				inhibit_gtk_themes = FALSE;
+			} else if ( ! inhibit_gtk_modules) {
+				/* on failure try again, this time without any modules
+				   which may be causing a crash */
+				inhibit_gtk_modules = TRUE;
+				ret = gdm_failsafe_question (d, question, echo);
+				inhibit_gtk_modules = FALSE;
+			}
+			return ret;
+		}
 
 		IGNORE_EINTR (bytes = read (p[0], buf, BUFSIZ-1));
 		if (bytes > 0) {
@@ -532,7 +616,7 @@ gdm_failsafe_yesno (GdmDisplay *d,
 	pid_t pid;
 	int p[2];
 
-	if (pipe (p) < 0)
+	if G_UNLIKELY (pipe (p) < 0)
 		return FALSE;
 
 	pid = gdm_fork_extra ();
@@ -541,7 +625,7 @@ gdm_failsafe_yesno (GdmDisplay *d,
 		GtkWidget *dlg;
 		char *loc;
 
-		setup_dialog (d, "gtk-failsafe-yesno", p[1], TRUE /* set_ids */);
+		setup_dialog (d, "gtk-failsafe-yesno", p[1], TRUE /* set_gdm_ids */, 0);
 
 		loc = gdm_locale_to_utf8 (question);
 
@@ -551,6 +635,7 @@ gdm_failsafe_yesno (GdmDisplay *d,
 					      GTK_BUTTONS_YES_NO,
 					      "%s",
 					      loc);
+		gtk_widget_set_events (dlg, GDK_ALL_EVENTS_MASK);
 		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 
 		sid = g_signal_lookup ("event",
@@ -589,12 +674,32 @@ gdm_failsafe_yesno (GdmDisplay *d,
 
 		_exit (0);
 	} else if (pid > 0) {
+		int status;
 		char buf[BUFSIZ];
 		int bytes;
 
 		IGNORE_EINTR (close (p[1]));
 
-		gdm_wait_for_extra (NULL);
+		gdm_wait_for_extra (&status);
+
+		if (dialog_failed (status)) {
+			gboolean ret = FALSE;
+			IGNORE_EINTR (close (p[0]));
+			if ( ! inhibit_gtk_themes) {
+				/* on failure try again, this time without any themes
+				   which may be causing a crash */
+				inhibit_gtk_themes = TRUE;
+				ret = gdm_failsafe_yesno (d, question);
+				inhibit_gtk_themes = FALSE;
+			} else if ( ! inhibit_gtk_modules) {
+				/* on failure try again, this time without any modules
+				   which may be causing a crash */
+				inhibit_gtk_modules = TRUE;
+				ret = gdm_failsafe_yesno (d, question);
+				inhibit_gtk_modules = FALSE;
+			}
+			return ret;
+		}
 
 		IGNORE_EINTR (bytes = read (p[0], buf, BUFSIZ-1));
 		if (bytes > 0) {
@@ -620,7 +725,7 @@ gdm_failsafe_ask_buttons (GdmDisplay *d,
 	pid_t pid;
 	int p[2];
 
-	if (pipe (p) < 0)
+	if G_UNLIKELY (pipe (p) < 0)
 		return -1;
 
 	pid = gdm_fork_extra ();
@@ -630,7 +735,7 @@ gdm_failsafe_ask_buttons (GdmDisplay *d,
 		GtkWidget *dlg;
 		char *loc;
 
-		setup_dialog (d, "gtk-failsafe-ask-buttons", p[1], TRUE /* set_ids */);
+		setup_dialog (d, "gtk-failsafe-ask-buttons", p[1], TRUE /* set_gdm_ids */, 0);
 
 		loc = gdm_locale_to_utf8 (question);
 
@@ -641,6 +746,7 @@ gdm_failsafe_ask_buttons (GdmDisplay *d,
 					      "%s",
 					      loc);
 		g_free (loc);
+		gtk_widget_set_events (dlg, GDK_ALL_EVENTS_MASK);
 		for (i = 0; but[i] != NULL; i++) {
 			loc = gdm_locale_to_utf8 (but[i]);
 			gtk_dialog_add_button (GTK_DIALOG (dlg),
@@ -684,12 +790,32 @@ gdm_failsafe_ask_buttons (GdmDisplay *d,
 
 		_exit (0);
 	} else if (pid > 0) {
+		int status;
 		char buf[BUFSIZ];
 		int bytes;
 
 		IGNORE_EINTR (close (p[1]));
 
-		gdm_wait_for_extra (NULL);
+		gdm_wait_for_extra (&status);
+
+		if (dialog_failed (status)) {
+			int ret = -1;
+			IGNORE_EINTR (close (p[0]));
+			if ( ! inhibit_gtk_themes) {
+				/* on failure try again, this time without any themes
+				   which may be causing a crash */
+				inhibit_gtk_themes = TRUE;
+				ret = gdm_failsafe_ask_buttons (d, question, but);
+				inhibit_gtk_themes = FALSE;
+			} else if ( ! inhibit_gtk_modules) {
+				/* on failure try again, this time without any modules
+				   which may be causing a crash */
+				inhibit_gtk_modules = TRUE;
+				ret = gdm_failsafe_ask_buttons (d, question, but);
+				inhibit_gtk_modules = FALSE;
+			}
+			return ret;
+		}
 
 		IGNORE_EINTR (bytes = read (p[0], buf, BUFSIZ-1));
 		if (bytes > 0) {
