@@ -99,7 +99,7 @@ gdm_connection_handler (GIOChannel *source,
 	if ( ! (cond & G_IO_IN)) 
 		return close_if_needed (conn, cond);
 
-	len = read (conn->fd, buf, sizeof (buf) -1);
+	IGNORE_EINTR (len = read (conn->fd, buf, sizeof (buf) -1));
 	if (len <= 0) {
 		return close_if_needed (conn, cond);
 	}
@@ -154,18 +154,18 @@ gdm_connection_write (GdmConnection *conn, const char *str)
 	g_return_val_if_fail (conn != NULL, FALSE);
 	g_return_val_if_fail (str != NULL, FALSE);
 
-	if ( ! conn->writable)
+	if G_UNLIKELY ( ! conn->writable)
 		return FALSE;
 
 #ifdef MSG_NOSIGNAL
-	ret = send (conn->fd, str, strlen (str), MSG_NOSIGNAL);
+	IGNORE_EINTR (ret = send (conn->fd, str, strlen (str), MSG_NOSIGNAL));
 #else
 	old_handler = signal (SIGPIPE, SIG_IGN);
-	ret = send (conn->fd, str, strlen (str), 0);
+	IGNORE_EINTR (ret = send (conn->fd, str, strlen (str), 0));
 	signal (SIGPIPE, old_handler);
 #endif
 
-	if (ret < 0)
+	if G_UNLIKELY (ret < 0)
 		return FALSE;
 	else
 		return TRUE;
@@ -186,10 +186,10 @@ gdm_socket_handler (GIOChannel *source,
 	if ( ! (cond & G_IO_IN)) 
 		return TRUE;
 
-	fd = accept (conn->fd,
-		     (struct sockaddr *)&addr,
-		     &addr_size);
-	if (fd < 0) {
+	IGNORE_EINTR (fd = accept (conn->fd,
+				   (struct sockaddr *)&addr,
+				   &addr_size));
+	if G_UNLIKELY (fd < 0) {
 		gdm_debug ("gdm_socket_handler: Rejecting connection");
 		return TRUE;
 	}
@@ -240,28 +240,52 @@ gdm_connection_open_unix (const char *sockname, mode_t mode)
 	GdmConnection *conn;
 	struct sockaddr_un addr;
 	int fd;
-
-	unlink (sockname);
+	int try_again_attempts = 1000;
 
 	fd = socket (AF_UNIX, SOCK_STREAM, 0);
-	if (fd < 0) {
+	if G_UNLIKELY (fd < 0) {
 		gdm_error (_("%s: Could not make socket"),
 			   "gdm_connection_open_unix");
 		return NULL;
 	}
 
+try_again:
+	/* this is all for creating sockets in /tmp/ safely */
+	IGNORE_EINTR (unlink (sockname));
+	if G_UNLIKELY (errno == EISDIR ||
+		       errno == EPERM) {
+		/* likely a directory, someone's playing tricks on us */
+		char *newname = NULL;
+		do {
+			g_free (newname);
+			newname = g_strdup_printf ("%s-renamed-%u",
+						   sockname,
+						   (guint)g_random_int ());
+		} while (access (newname, F_OK) == 0);
+		IGNORE_EINTR (rename (sockname, newname));
+		if G_UNLIKELY (errno != 0)
+			try_again_attempts = 0;
+		g_free (newname);
+	} else if G_UNLIKELY (errno != 0) {
+		try_again_attempts = 0;
+	}
+
 	memset(&addr, 0, sizeof(addr));
 	strcpy (addr.sun_path, sockname);
 	addr.sun_family = AF_UNIX;
-	if (bind (fd,
-		  (struct sockaddr *) &addr, sizeof (addr)) < 0) {
+	if G_UNLIKELY (bind (fd,
+			     (struct sockaddr *) &addr, sizeof (addr)) < 0) {
 		gdm_error (_("%s: Could not bind socket"),
 			   "gdm_connection_open_unix");
-		close (fd);
+		try_again_attempts --;
+		/* someone is being evil on us */
+		if (errno == EADDRINUSE && try_again_attempts >= 0)
+			goto try_again;
+		IGNORE_EINTR (close (fd));
 		return NULL;
 	}
 
-	chmod (sockname, mode);
+	IGNORE_EINTR (chmod (sockname, mode));
 
 	conn = g_new0 (GdmConnection, 1);
 	conn->close_level = 0;
@@ -295,7 +319,7 @@ gdm_connection_open_fd (int fd)
 	GIOChannel *unixchan;
 	GdmConnection *conn;
 
-	g_return_val_if_fail (fd < 0, NULL);
+	g_return_val_if_fail (fd >= 0, NULL);
 
 	conn = g_new0 (GdmConnection, 1);
 	conn->close_level = 0;
@@ -315,11 +339,9 @@ gdm_connection_open_fd (int fd)
 	conn->source = g_io_add_watch_full
 		(unixchan, G_PRIORITY_DEFAULT,
 		 G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
-		 gdm_socket_handler, conn, NULL);
+		 gdm_connection_handler, conn, NULL);
 	g_io_channel_unref (unixchan);
 
-	listen (fd, 5);
-
 	return conn;
 }
 
@@ -330,9 +352,9 @@ gdm_connection_open_fifo (const char *fifo, mode_t mode)
 	GdmConnection *conn;
 	int fd;
 
-	unlink (fifo);
+	IGNORE_EINTR (unlink (fifo));
 
-	if (mkfifo (fifo, 0660) < 0) {
+	if G_UNLIKELY (mkfifo (fifo, 0660) < 0) {
 		gdm_error (_("%s: Could not make FIFO"),
 			   "gdm_connection_open_fifo");
 		return NULL;
@@ -340,13 +362,13 @@ gdm_connection_open_fifo (const char *fifo, mode_t mode)
 
 	fd = open (fifo, O_RDWR); /* Open with write to avoid EOF */
 
-	if (fd < 0) {
+	if G_UNLIKELY (fd < 0) {
 		gdm_error (_("%s: Could not open FIFO"),
 			   "gdm_connection_open_fifo");
 		return NULL;
 	}
 
-	chmod (fifo, mode);
+	IGNORE_EINTR (chmod (fifo, mode));
 
 	conn = g_new0 (GdmConnection, 1);
 	conn->close_level = 0;
@@ -455,7 +477,7 @@ gdm_connection_close (GdmConnection *conn)
 	}
 
 	if (conn->fd > 0) {
-		close (conn->fd);
+		IGNORE_EINTR (close (conn->fd));
 		conn->fd = -1;
 	}
 
@@ -468,7 +490,7 @@ gdm_connection_close (GdmConnection *conn)
 void
 gdm_connection_set_close_notify (GdmConnection *conn,
 				 gpointer close_data,
-				 gpointer close_notify)
+				 GDestroyNotify close_notify)
 {
 	g_return_if_fail (conn != NULL);
 
