@@ -137,7 +137,6 @@ gchar *GdmGtkModulesList = NULL;
 gchar *GdmChooser = NULL;
 gchar *GdmLogDir = NULL;
 gchar *GdmDisplayInit = NULL;
-gchar *GdmSessionDir = NULL;
 gchar *GdmPostLogin = NULL;
 gchar *GdmPreSession = NULL;
 gchar *GdmPostSession = NULL;
@@ -294,11 +293,6 @@ gdm_config_parse (void)
     displays = NULL;
     high_display_num = 0;
 
-#if defined(_SCO_DS) || defined(__UNIXWARE__) || defined(__USLC__)
-    gdm_error ("SCO OS detected!!!  I will sleep for 5 seconds in protest.  This message can be disabled by SCO stopping to sue people.");
-    sleep (5);
-#endif
-
     IGNORE_EINTR (r = stat (GDM_CONFIG_FILE, &statbuf));
     if (r < 0) {
 	    gdm_error (_("%s: No configuration file: %s. Using defaults."),
@@ -326,7 +320,6 @@ gdm_config_parse (void)
     GdmKillInitClients = ve_config_get_bool (cfg, GDM_KEY_KILLIC);
     GdmLogDir= ve_config_get_string (cfg, GDM_KEY_LOGDIR);
     GdmPidFile = ve_config_get_string (cfg, GDM_KEY_PIDFILE);
-    GdmSessionDir = ve_config_get_string (cfg, GDM_KEY_SESSDIR);
     GdmPostLogin = ve_config_get_string (cfg, GDM_KEY_POSTLOGIN);
     GdmPreSession = ve_config_get_string (cfg, GDM_KEY_PRESESS);
     GdmPostSession = ve_config_get_string (cfg, GDM_KEY_POSTSESS);
@@ -612,7 +605,7 @@ gdm_config_parse (void)
     /* Lookup user and groupid for the gdm user */
     pwent = getpwnam (GdmUser);
 
-    if G_UNLIKELY (pwent == NULL) {
+    if G_UNLIKELY (pwent == NULL && 0 /* FIXME: this is evil! */) {
 	    gdm_error (_("%s: Can't find the gdm user (%s). Trying 'nobody'!"), "gdm_config_parse", GdmUser);
 	    g_free (GdmUser);
 	    GdmUser = g_strdup ("nobody");
@@ -648,7 +641,7 @@ gdm_config_parse (void)
 
     grent = getgrnam (GdmGroup);
 
-    if G_UNLIKELY (grent == NULL) {
+    if G_UNLIKELY (grent == NULL && 0 /* FIXME: this is evil! */) {
 	    gdm_error (_("%s: Can't find the gdm group (%s). Trying 'nobody'!"), "gdm_config_parse", GdmUser);
 	    g_free (GdmGroup);
 	    GdmGroup = g_strdup ("nobody");
@@ -832,7 +825,7 @@ gdm_daemonify (void)
 	    errno = 0;
 	    fprintf (pf, "%d\n", (int)pid);
 	    fclose (pf);
-	    if (errno != 0) {
+	    if G_UNLIKELY (errno != 0) {
 		    /* FIXME: how to handle this? */
 		    gdm_fdprintf (2, "Cannot write PID file %s, possibly out of diskspace.  Error: %s\n",
 				  GdmPidFile, strerror (errno));
@@ -840,7 +833,7 @@ gdm_daemonify (void)
 			       GdmPidFile, strerror (errno));
 
 	    }
-	} else if (errno != 0) {
+	} else if G_UNLIKELY (errno != 0) {
 	    /* FIXME: how to handle this? */
 	    gdm_fdprintf (2, "Cannot write PID file %s, possibly out of diskspace.  Error: %s\n",
 			  GdmPidFile, strerror (errno));
@@ -877,6 +870,9 @@ gdm_start_first_unborn_local (int delay)
 {
 	GSList *li;
 
+	/* tickle the random stuff */
+	gdm_random_tick ();
+
 	for (li = displays; li != NULL; li = li->next) {
 		GdmDisplay *d = li->data;
 
@@ -1804,6 +1800,7 @@ main (int argc, char *argv[])
     int nextopt;
     const char *charset;
 
+    /* semi init pseudorandomness */
     gdm_random_tick ();
 
     gdm_main_pid = getpid ();
