@@ -313,6 +313,24 @@ gdm_saveenv (void)
 	}
 }
 
+const char *
+gdm_saved_getenv (const char *var)
+{
+	int len;
+	GList *li;
+
+	len = strlen (var);
+
+	for (li = stored_env; li != NULL; li = li->next) {
+		const char *e = li->data;
+		if (strncmp (var, e, len) == 0 &&
+		    e[len] == '=') {
+			return &(e[len+1]);
+		}
+	}
+	return NULL;
+}
+
 /* leaks */
 void
 gdm_restoreenv (void)
@@ -1221,6 +1239,13 @@ gdm_unset_signals (void)
 	gdm_signal_default (SIGPIPE);
 	gdm_signal_default (SIGALRM);
 	gdm_signal_default (SIGHUP);
+	gdm_signal_default (SIGABRT);
+#ifdef SIGXFSZ
+	gdm_signal_default (SIGXFSZ);
+#endif
+#ifdef SIGXCPU
+	gdm_signal_default (SIGXCPU);
+#endif
 }
 
 void
