@@ -24,19 +24,23 @@
 /* Wipe cookie files */
 void		gdm_server_wipe_cookies	(GdmDisplay *disp);
 
+/* Wipe X server's butt */
+void		gdm_server_whack_lockfile (GdmDisplay *disp);
+
 gboolean	gdm_server_start	(GdmDisplay *d,
 					 gboolean treat_as_flexi,
 					 int min_flexi_disp,
 					 int flexi_retries);
 void		gdm_server_stop		(GdmDisplay *d);
-void		gdm_server_reinit	(GdmDisplay *d);
+gboolean	gdm_server_reinit	(GdmDisplay *d);
 GdmDisplay *	gdm_server_alloc	(gint id,
 					 const gchar *command);
 void		gdm_server_whack_clients (GdmDisplay *disp);
 void		gdm_server_checklog	(GdmDisplay *disp);
 
 char **		gdm_server_resolve_command_line (GdmDisplay *disp,
-						 gboolean resolve_handled);
+						 gboolean resolve_flags,
+						 const char *vtarg);
 
 
 
