@@ -32,6 +32,14 @@ gboolean gdm_slave_should_complain (void);
 void	 gdm_slave_send		(const char *str, gboolean wait_for_ack);
 void	 gdm_slave_send_num	(const char *opcode, long num);
 void     gdm_slave_send_string	(const char *opcode, const char *str);
+gboolean gdm_slave_final_cleanup (void);
+
+void     gdm_slave_whack_temp_auth_file (void);
+
+gboolean gdm_slave_check_user_wants_to_log_in (const char *user);
+
+/* This is the slave child handler so that we can chain to it from elsewhere */
+void	 gdm_slave_child_handler (int sig);
 
 #endif /* GDM_SLAVE_H */
 
