@@ -6,6 +6,7 @@
 #include "greeter.h"
 #include "greeter_item_pam.h"
 #include "greeter_item_ulist.h"
+#include "greeter_item_timed.h"
 #include "greeter_parser.h"
 #include "greeter_configuration.h"
 #include "gdm.h"
@@ -24,6 +25,57 @@ gboolean require_quarter = FALSE;
 
 extern gboolean greeter_probably_login_prompt;
 
+static char *
+break_at (const char *orig, int columns)
+{
+  PangoLogAttr *attrs;
+  int n_chars;
+  GString *str;
+  int i;
+  int in_current_row;
+  const char *p;
+  
+  str = g_string_new (NULL);
+  n_chars = g_utf8_strlen (orig, -1);
+
+  attrs = g_new0 (PangoLogAttr, n_chars + 1);
+  pango_get_log_attrs (orig, -1,
+                       0, gtk_get_default_language (),
+                       attrs, n_chars + 1);  
+
+  in_current_row = 0;
+  i = 0;
+  p = orig;
+  while (i < n_chars)
+    {
+      gunichar ch;
+
+      ch = g_utf8_get_char (p);
+      
+      /* Broken algorithm for simplicity; we just break
+       * at the first place we can within 10 of the end
+       */
+      
+      if (in_current_row > (columns - 10) &&
+          attrs[i].is_line_break)
+        {
+          in_current_row = 0;
+          g_string_append_unichar (str, '\n');
+        }
+
+      ++in_current_row;
+      g_string_append_unichar (str, ch);
+      
+      p = g_utf8_next_char (p);
+      ++i;
+    }
+  
+  g_free (attrs);
+
+  return g_string_free (str, FALSE);
+}
+
+
 void
 greeter_item_pam_set_user (const char *user)
 {
@@ -65,12 +117,23 @@ user_pw_activate (GtkEntry *entry, GreeterItemInfo *info)
   str = gtk_entry_get_text (GTK_ENTRY (entry));
   if (greeter_probably_login_prompt &&
       /* evilness */
-      evil (entry, str)) {
-	  /* obviously being 100% reliable is not an issue for
-	     this test */
-	  gtk_entry_set_text (GTK_ENTRY (entry), "");
-	  return;
-  }
+      evil (entry, str))
+    {
+      /* obviously being 100% reliable is not an issue for
+         this test */
+      gtk_entry_set_text (GTK_ENTRY (entry), "");
+      return;
+    }
+
+  if (greeter_probably_login_prompt &&
+      ve_string_empty (str) &&
+      greeter_item_timed_is_timed ())
+    {
+      /* timed interruption */
+      printf ("%c%c%c\n", STX, BEL, GDM_INTERRUPT_TIMED_LOGIN);
+      fflush (stdout);
+      return;
+    }
   
   gtk_widget_set_sensitive (GTK_WIDGET (entry), FALSE);
 
@@ -156,9 +219,21 @@ greeter_item_pam_prompt (const char *message,
   entry_info = greeter_lookup_id ("user-pw-entry");
 
   if (conversation_info)
-    g_object_set (G_OBJECT (conversation_info->item),
-		  "text", message,
-		  NULL);
+    {
+      char *text;
+
+      /* This is a bad hack that I added because
+       * the canvas text item doesn't support
+       * text wrapping...
+       */
+      text = break_at (message, 50);
+
+      g_object_set (G_OBJECT (conversation_info->item),
+		    "text", text,
+		    NULL);
+
+      g_free (text);
+    }
   
   if (entry_info && entry_info->item &&
       GNOME_IS_CANVAS_WIDGET (entry_info->item))
@@ -190,6 +265,14 @@ greeter_item_pam_message (const char *message)
 
   if (message_info)
     {
+      char *text;
+
+      /* This is a bad hack that I added because
+       * the canvas text item doesn't support
+       * text wrapping...
+       */
+      text = break_at (message, 50);
+
       /* HAAAAAAACK.  Sometimes pam sends many many messages, SO
        * we try to collect them until the next prompt or reset or
        * whatnot */
@@ -208,13 +291,15 @@ greeter_item_pam_message (const char *message)
 	    }
 	  else
 	    g_object_set (G_OBJECT (message_info->item),
-			  "text", message,
+			  "text", text,
 			  NULL);
 	}
       else
 	g_object_set (G_OBJECT (message_info->item),
-		      "text", message,
+		      "text", text,
 		      NULL);
+
+      g_free (text);
     }
   replace_msg = FALSE;
 }
@@ -233,56 +318,6 @@ error_clear (gpointer data)
 	return FALSE;
 }
 
-static char *
-break_at (const char *orig, int columns)
-{
-  PangoLogAttr *attrs;
-  int n_chars;
-  GString *str;
-  int i;
-  int in_current_row;
-  const char *p;
-  
-  str = g_string_new (NULL);
-  n_chars = g_utf8_strlen (orig, -1);
-
-  attrs = g_new0 (PangoLogAttr, n_chars + 1);
-  pango_get_log_attrs (orig, -1,
-                       0, gtk_get_default_language (),
-                       attrs, n_chars + 1);  
-
-  in_current_row = 0;
-  i = 0;
-  p = orig;
-  while (i < n_chars)
-    {
-      gunichar ch;
-
-      ch = g_utf8_get_char (p);
-      
-      /* Broken algorithm for simplicity; we just break
-       * at the first place we can within 10 of the end
-       */
-      
-      if (in_current_row > (columns - 10) &&
-          attrs[i].is_line_break)
-        {
-          in_current_row = 0;
-          g_string_append_unichar (str, '\n');
-        }
-
-      ++in_current_row;
-      g_string_append_unichar (str, ch);
-      
-      p = g_utf8_next_char (p);
-      ++i;
-    }
-  
-  g_free (attrs);
-
-  return g_string_free (str, FALSE);
-}
-
 void
 greeter_item_pam_error (const char *message)
 {
@@ -350,6 +385,7 @@ greeter_item_pam_leftover_messages (void)
 				   GTK_DIALOG_MODAL /* flags */,
 				   GTK_MESSAGE_INFO,
 				   GTK_BUTTONS_OK,
+				   FALSE /* markup */,
 				   oldtext,
 				   /* avoid warning */ "%s", "");
 	  gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
