@@ -43,8 +43,8 @@ greeter_login_list_lookup (GSList *l, const gchar *data)
 {
     GSList *list = l;
 
-    if (!list || !data)
-	return(FALSE);
+    if (list == NULL || data == NULL)
+	return FALSE;
 
     /* FIXME: Hack, will support these builtin types later */
     if (strcmp (data, GDM_SESSION_DEFAULT ".desktop") == 0 ||
@@ -56,12 +56,12 @@ greeter_login_list_lookup (GSList *l, const gchar *data)
     while (list) {
 
 	if (strcmp (list->data, data) == 0)
-	    return (TRUE);
+	    return TRUE;
 	
 	list = list->next;
     }
 
-    return (FALSE);
+    return FALSE;
 }
 
 static const char *
@@ -92,7 +92,7 @@ greeter_session_lookup (const char *saved_session)
   if ( ! (saved_session != NULL &&
 	  strcmp ("(null)", saved_session) != 0 &&
 	  saved_session[0] != '\0')) {
-    /* If "Last" is chosen run Default,
+    /* If "Last" is chosen run default,
      * else run user's current selection */
     if (current_session == NULL || strcmp (current_session, LAST_SESSION) == 0)
       session = g_strdup (default_session);
@@ -121,7 +121,7 @@ greeter_session_lookup (const char *saved_session)
 				   "future sessions?"),
 				 session_name (saved_session),
 				 session_name (default_session));	    
-	  save_session = greeter_query (msg, _("Make _Default"), _("Just _Log In"));
+	  save_session = greeter_query (msg, FALSE /* markup */, _("Make _Default"), _("Just _Log In"));
 	  g_free (msg);
 	}
     }
@@ -132,9 +132,8 @@ greeter_session_lookup (const char *saved_session)
       /* User's saved session is not the chosen one */
       if (strcmp (session, GDM_SESSION_FAILSAFE_GNOME) == 0 ||
 	  strcmp (session, GDM_SESSION_FAILSAFE_XTERM) == 0 ||
-	  /* bad hack, "Failsafe" is just a name in the session dir */
-	  strcmp (session, "Failsafe.desktop") == 0 ||
-	  strcmp (session, "Failsafe") == 0)
+	  g_ascii_strcasecmp (session, GDM_SESSION_FAILSAFE ".desktop") == 0 ||
+	  g_ascii_strcasecmp (session, GDM_SESSION_FAILSAFE) == 0)
 	{
 	  save_session = FALSE;
 	}
@@ -152,9 +151,10 @@ greeter_session_lookup (const char *saved_session)
 				     session_name (session),
 				     session_name (saved_session),
 				     session_name (session));
-	      save_session = greeter_query (msg, _("Make _Default"), _("Just For _This Session"));
+	      save_session = greeter_query (msg, FALSE /* markup */, _("Make _Default"), _("Just For _This Session"));
 	    }
-	  else if (strcmp (session, "Xclients.desktop") != 0 &&
+	  else if (strcmp (session, default_session) != 0 &&
+		   strcmp (session, saved_session) != 0 &&
 		   strcmp (session, LAST_SESSION) != 0)
 	    {
 	      /* if !GdmShowLastSession then our saved session is
@@ -162,16 +162,19 @@ greeter_session_lookup (const char *saved_session)
 	       * and the relevant thing is the saved session
 	       * in .Xclients
 	       */
-	      msg = g_strdup_printf (_("You have chosen %s for this "
-				       "session.\nIf you wish to make %s "
-				       "the default for future sessions,\n"
-				       "run the 'switchdesk' utility\n"
-				       "(System->Desktop Switching Tool from "
-				       "the panel menu)."),
-				     session_name (session),
-				     session_name (session));
+	      if (access ("/usr/bin/switchdesk", F_OK) == 0)
+	        {
+	          msg = g_strdup_printf (_("You have chosen %s for this "
+				           "session.\nIf you wish to make %s "
+				           "the default for future sessions,\n"
+				           "run the 'switchdesk' utility\n"
+					   "(System->Desktop Switching Tool from "
+					   "the panel menu)."),
+					 session_name (session),
+					 session_name (session));
+		  greeter_message (msg);
+		}
 	      save_session = FALSE;
-	      greeter_message (msg);
 	    }
 	  g_free (msg);
 	}
@@ -203,6 +206,9 @@ greeter_session_init (void)
   GtkRequisition req;
   char *s;
   int num = 1;
+  int i;
+  char **vec;
+  gboolean some_dir_exists = FALSE;
 
   g_free (current_session);
   current_session = NULL;
@@ -273,134 +279,157 @@ greeter_session_init (void)
       gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 4);
       gtk_widget_show (radio);
     }
-    
+
+    vec = g_strsplit (GdmSessionDir, ":", -1);
+    for (i = 0; vec != NULL && vec[i] != NULL; i++)
+      {
+        const char *dir = vec[i];
+
+	/* Check that session dir is readable */
+	if G_UNLIKELY (dir == NULL || access (dir, R_OK|X_OK) != 0)
+		continue;
+
+	some_dir_exists = TRUE;
+
+	/* Read directory entries in session dir */
+	sessdir = opendir (dir);
+
+	if G_LIKELY (sessdir != NULL)
+		dent = readdir (sessdir);
+	else
+		dent = NULL;
+
+	while (dent != NULL) {
+		VeConfig *cfg;
+		char *exec;
+		char *name;
+		char *comment;
+		char *label;
+		char *tryexec;
+		char *ext;
+
+		/* ignore everything bug the .desktop files */
+		ext = strstr (dent->d_name, ".desktop");
+		if (ext == NULL ||
+		    strcmp (ext, ".desktop") != 0) {
+			dent = readdir (sessdir);
+			continue;
+		}
+
+		/* already found this session, ignore */
+		if (g_hash_table_lookup (sessnames, dent->d_name) != NULL) {
+			dent = readdir (sessdir);
+			continue;
+		}
+
+		s = g_strconcat (dir, "/", dent->d_name, NULL);
+		cfg = ve_config_new (s);
+		g_free (s);
+
+		if (ve_config_get_bool (cfg, "Desktop Entry/Hidden=false")) {
+			g_hash_table_insert (sessnames, g_strdup (dent->d_name), "foo");
+			ve_config_destroy (cfg);
+			dent = readdir (sessdir);
+			continue;
+		}
+
+		tryexec = ve_config_get_string (cfg, "Desktop Entry/TryExec");
+		if ( ! ve_string_empty (tryexec)) {
+			char *full = g_find_program_in_path (tryexec);
+			if (full == NULL) {
+				g_hash_table_insert (sessnames, g_strdup (dent->d_name), "foo");
+				g_free (tryexec);
+				ve_config_destroy (cfg);
+				dent = readdir (sessdir);
+				continue;
+			}
+			g_free (full);
+		}
+		g_free (tryexec);
+
+		exec = ve_config_get_string (cfg, "Desktop Entry/Exec");
+		name = ve_config_get_translated_string (cfg, "Desktop Entry/Name");
+		comment = ve_config_get_translated_string (cfg, "Desktop Entry/Comment");
+
+		ve_config_destroy (cfg);
+
+		if G_UNLIKELY (ve_string_empty (exec) || ve_string_empty (name)) {
+			g_hash_table_insert (sessnames, g_strdup (dent->d_name), "foo");
+			g_free (exec);
+			g_free (name);
+			g_free (comment);
+			dent = readdir (sessdir);
+			continue;
+		}
+
+		if (num < 10)
+			label = g_strdup_printf ("_%d. %s", num, name);
+		else
+			label = g_strdup (name);
+		num ++;
+
+		radio = gtk_radio_button_new_with_mnemonic (session_group, label);
+		g_free (label);
+		g_object_set_data_full (G_OBJECT (radio),
+					SESSION_NAME,
+					g_strdup (dent->d_name),
+					(GDestroyNotify) g_free);
+		session_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
+		gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 4);
+		gtk_widget_show (radio);
+
+		if ( ! ve_string_empty (comment))
+			gtk_tooltips_set_tip
+				(tooltips, GTK_WIDGET (radio), comment, NULL);
+
+
+		sessions = g_slist_append (sessions, g_strdup (dent->d_name));
+
+		/* if we found the default session */
+		if ( ! ve_string_empty (GdmDefaultSession) &&
+		     strcmp (dent->d_name, GdmDefaultSession) == 0) {
+			g_free (default_session);
+			default_session = g_strdup (dent->d_name);
+			searching_for_default = FALSE;
+		}
+
+		/* if there is a session called default */
+		if (searching_for_default &&
+		    g_ascii_strcasecmp (dent->d_name, "default.desktop") == 0) {
+			g_free (default_session);
+			default_session = g_strdup (dent->d_name);
+		}
+
+		if (searching_for_default &&
+		    g_ascii_strcasecmp (dent->d_name, "gnome.desktop") == 0) {
+			/* Just in case there is no default session and
+			 * no default link, make Gnome the default */
+			if (default_session == NULL)
+				default_session = g_strdup (dent->d_name);
+		}
+
+		g_hash_table_insert (sessnames, g_strdup (dent->d_name), name);
+
+		g_free (exec);
+		g_free (comment);
+
+		dent = readdir (sessdir);
+	}
+
+	if (sessdir != NULL)
+		closedir (sessdir);
+      }
+
+    g_strfreev (vec);
+
     /* Check that session dir is readable */
-    if G_UNLIKELY (GdmSessionDir == NULL ||
-		   access (GdmSessionDir, R_OK|X_OK))
+    if G_UNLIKELY ( ! some_dir_exists)
       {
 	syslog (LOG_ERR, _("%s: Session directory %s not found!"), "gdm_login_session_init", ve_sure_string (GdmSessionDir));
 	session_dir_whacked_out = TRUE;
 	GdmShowXtermFailsafeSession = TRUE;
       }
 
-    /* Read directory entries in session dir */
-    if G_UNLIKELY (GdmSessionDir == NULL)
-	    sessdir = NULL;
-    else
-	    sessdir = opendir (GdmSessionDir);
-
-    if G_LIKELY (sessdir != NULL)
-	    dent = readdir (sessdir);
-    else
-	    dent = NULL;
-
-    while (dent != NULL) {
-	    VeConfig *cfg;
-	    char *exec;
-	    char *name;
-	    char *comment;
-	    char *label;
-	    char *tryexec;
-
-	    /* ignore everything bug the .desktop files */
-	    if (strstr (dent->d_name, ".desktop") == NULL) {
-		    dent = readdir (sessdir);
-		    continue;
-	    }
-
-	    s = g_strconcat (GdmSessionDir, "/", dent->d_name, NULL);
-	    cfg = ve_config_new (s);
-	    g_free (s);
-
-	    if (ve_config_get_bool (cfg, "Desktop Entry/Hidden=false")) {
-		    ve_config_destroy (cfg);
-		    dent = readdir (sessdir);
-		    continue;
-	    }
-
-	    tryexec = ve_config_get_string (cfg, "Desktop Entry/TryExec");
-	    if ( ! ve_string_empty (tryexec)) {
-		    char *full = g_find_program_in_path (tryexec);
-		    if (full == NULL) {
-			    g_free (tryexec);
-			    ve_config_destroy (cfg);
-			    dent = readdir (sessdir);
-			    continue;
-		    }
-		    g_free (full);
-	    }
-	    g_free (tryexec);
-
-	    exec = ve_config_get_string (cfg, "Desktop Entry/Exec");
-	    name = ve_config_get_translated_string (cfg, "Desktop Entry/Name");
-	    comment = ve_config_get_translated_string (cfg, "Desktop Entry/Comment");
-
-	    ve_config_destroy (cfg);
-
-	    if G_UNLIKELY (ve_string_empty (exec) || ve_string_empty (name)) {
-		    g_free (exec);
-		    g_free (name);
-		    g_free (comment);
-		    dent = readdir (sessdir);
-		    continue;
-	    }
-
-	    if (num < 10)
-		    label = g_strdup_printf ("_%d. %s", num, name);
-	    else
-		    label = g_strdup (name);
-	    num ++;
-
-	    radio = gtk_radio_button_new_with_mnemonic (session_group, label);
-	    g_free (label);
-	    g_object_set_data_full (G_OBJECT (radio),
-				    SESSION_NAME,
-				    g_strdup (dent->d_name),
-				    (GDestroyNotify) g_free);
-	    session_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
-	    gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 4);
-	    gtk_widget_show (radio);
-
-	    if ( ! ve_string_empty (comment))
-		    gtk_tooltips_set_tip
-			    (tooltips, GTK_WIDGET (radio), comment, NULL);
-
-
-	    sessions = g_slist_append (sessions, g_strdup (dent->d_name));
-
-	    /* if we found the default session */
-	    if ( ! ve_string_empty (GdmDefaultSession) &&
-		 strcmp (dent->d_name, GdmDefaultSession) == 0) {
-		    g_free (default_session);
-		    default_session = g_strdup (dent->d_name);
-		    searching_for_default = FALSE;
-	    }
-
-	    /* if there is a session called Default */
-	    if (searching_for_default &&
-		g_ascii_strcasecmp (dent->d_name, "Default.desktop") == 0) {
-		    g_free (default_session);
-		    default_session = g_strdup (dent->d_name);
-	    }
-
-	    if (searching_for_default &&
-		g_ascii_strcasecmp (dent->d_name, "Gnome.desktop") == 0) {
-		    /* Just in case there is no Default session and
-		     * no default link, make Gnome the default */
-		    if (default_session == NULL)
-			    default_session = g_strdup (dent->d_name);
-	    }
-
-	    g_hash_table_insert (sessnames, g_strdup (dent->d_name), name);
-
-	    g_free (exec);
-	    g_free (comment);
-
-	    dent = readdir (sessdir);
-    }
-
-    if (sessdir != NULL)
-	    closedir (sessdir);
 
     if G_UNLIKELY (sessions == NULL)
       {
