@@ -378,6 +378,10 @@ find_or_make_section (VeConfig *config,
 	p = strchr (dkey, '/');
 	if (p == NULL) {
 		*key_name = dkey;
+		/* we're adding to the root which is completely
+		   empty so add a blank line to separate it */
+		if (make_new && config->root->lines == NULL)
+			add_text_line (config, config->root, "");
 		return config->root;
 	}
 	*p = '\0';
@@ -397,7 +401,9 @@ find_or_make_section (VeConfig *config,
 
 	/* Add an empty line on end of last section */
 	last_sect = find_last_section (config);
-	add_text_line (config, last_sect, "");
+	if (last_sect != config->root ||
+	    config->root->lines != NULL)
+		add_text_line (config, last_sect, "");
 
 	section = new_section (config, dkey);
 	config->sections = g_list_append (config->sections, section);
@@ -576,7 +582,7 @@ ve_config_get_translated_string (VeConfig *config,
 		def++;
 	}
 
-	for (li = gnome_i18n_get_language_list ("LC_MESSAGES");
+	for (li = ve_i18n_get_language_list ("LC_MESSAGES");
 	     li != NULL;
 	     li = li->next) {
 		char *full = g_strdup_printf ("%s[%s]", dkey, (char *)li->data);
@@ -835,3 +841,62 @@ ve_config_delete_translations (VeConfig *config, const char *key)
 	}
 	g_list_free (lines);
 }
+
+GList *
+ve_config_get_sections (VeConfig *config)
+{
+	GList *li;
+	GList *list;
+
+	g_return_val_if_fail (config != NULL, NULL);
+
+	list = NULL;
+
+	for (li = config->sections; li != NULL; li = li->next) {
+		VeSection *section = li->data;
+		list = g_list_prepend (list, g_strdup (section->name));
+	}
+	return g_list_reverse (list);
+}
+
+GList *
+ve_config_get_keys (VeConfig *config, const char *section)
+{
+	VeSection *sec;
+	GList *li;
+	GList *list;
+
+	g_return_val_if_fail (config != NULL, NULL);
+	g_return_val_if_fail (section != NULL, NULL);
+
+	sec = find_section (config, section);
+	if (sec == NULL)
+		return NULL;
+
+	list = NULL;
+
+	for (li = sec->lines; li != NULL; li = li->next) {
+		VeLine *ll = li->data;
+		if (ll->type == VE_LINE_KEY &&
+		    ! ve_string_empty (ll->key))
+			list = g_list_prepend (list, g_strdup (ll->key));
+	}
+	return g_list_reverse (list);
+}
+
+void
+ve_config_free_list_of_strings (GList *list)
+{
+	GList *li;
+
+	if (list == NULL)
+		return;
+
+	for (li = list; li != NULL; li = li->next) {
+		g_free (li->data);
+		li->data = NULL;
+	}
+
+	g_list_free (list);
+}
+
